package com.link.core.converter.i18n;

import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import javassist.ClassClassPath;
import javassist.ClassPool;
import javassist.CtClass;
import javassist.CtMethod;
import javassist.NotFoundException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class Resources {
   private static final Logger logger = LoggerFactory.getLogger(Resources.class);
   private static final Map<String, Object> objectMap = new HashMap();

   public static <T> T create(Class<T> constIntf) {
      return create(constIntf, (String)null, (Locale)null, (ClassLoader)null);
   }

   public static <T> T create(Class<T> constIntf, String resourceBaseName) {
      return create(constIntf, resourceBaseName, (Locale)null, (ClassLoader)null);
   }

   public static <T> T create(Class<T> constIntf, String resourceBaseName, Locale locale, ClassLoader classLoader) {
      if (constIntf == null) {
         return null;
      } else {
         if (resourceBaseName == null) {
            resourceBaseName = constIntf.getName().replace('.', '/');
         }

         if (locale == null) {
            locale = Locale.getDefault();
         }

         if (classLoader == null) {
            classLoader = constIntf.getClassLoader();
         }

         String key = constIntf.getName() + "." + resourceBaseName;
         Object object = objectMap.get(key);
         if (object != null) {
            return object;
         } else {
            ResourceBundle bundle = getResourceBundle(resourceBaseName, locale, classLoader);
            T impl = createImpl(constIntf, bundle, resourceBaseName);
            if (impl != null) {
               objectMap.put(key, impl);
            }

            return impl;
         }
      }
   }

   private static ResourceBundle getResourceBundle(String resourceBaseName, Locale locale, ClassLoader classLoader) {
      try {
         return ResourceBundle.getBundle(resourceBaseName, locale, classLoader);
      } catch (MissingResourceException var4) {
         return null;
      }
   }

   private static <T> T createImpl(Class<T> constIntf, ResourceBundle bundle, String resourceBaseName) {
      assert constIntf != null;

      try {
         ClassPool cp = ClassPool.getDefault();
         String className = constIntf.getName() + "$" + resourceBaseName.replace('/', '_') + "$javassist_impl";
         CtClass impl = cp.makeClass(className);
         CtClass intf = getCtClass(cp, constIntf);
         CtClass assist = cp.get(ResourcesAssist.class.getName());
         impl.setSuperclass(assist);
         impl.addInterface(intf);
         Method[] ims = constIntf.getMethods();
         Method[] var9 = ims;
         int var10 = ims.length;

         for(int var11 = 0; var11 < var10; ++var11) {
            Method im = var9[var11];
            if (!im.getDeclaringClass().equals(BaseResources.class)) {
               String value = ensureValue(bundle, im);
               CtMethod m = new CtMethod(cp.get(String.class.getName()), im.getName(), new CtClass[0], impl);
               m.setModifiers(1);
               m.setBody("{return \"" + value + "\";}");
               impl.addMethod(m);
            }
         }

         Class cls = cp.toClass(impl);
         T object = cls.newInstance();
         ((ResourcesAssist)object).setResourceBundle(bundle);
         return object;
      } catch (Exception var15) {
         Exception e = var15;
         logger.error("创建资源访问类失败。", e);
         return null;
      }
   }

   private static CtClass getCtClass(ClassPool cp, Class cls) throws NotFoundException {
      assert cp != null;

      assert cls != null;

      try {
         return cp.get(cls.getName());
      } catch (NotFoundException var3) {
         cp.insertClassPath(new ClassClassPath(cls));
         return cp.get(cls.getName());
      }
   }

   private static String ensureValue(ResourceBundle bundle, Method im) {
      String key = im.getName();
      String value = key;
      if (bundle != null && bundle.containsKey(key)) {
         value = bundle.getString(key);
      } else if (im.isAnnotationPresent(DefaultStringValue.class)) {
         DefaultStringValue def = (DefaultStringValue)im.getAnnotation(DefaultStringValue.class);
         value = def.value();
      }

      StringBuffer sb = new StringBuffer();
      char last = 0;

      for(int index = 0; index < value.length(); ++index) {
         char ch = value.charAt(index);
         if (ch == '"' && (index == 0 || last != '\\')) {
            sb.append('\\');
         }

         sb.append(ch);
         last = ch;
      }

      return sb.toString();
   }
}
